from valipy.myFunctions import Valipy, InputValidationException
from valipy.exceptions import InputIsNegativeException, InputIsPositiveException

def test_valipy():
    assert Valipy(22).isInt().isEven().isPositive().validate() == False

x = Valipy('test text 22').isType(str).isMinLength(5).itHasSpaces().itIncludesNumbers().validate()
print(x)





from typing import Any, Self, Union, Type, Callable, Literal
from .exceptions import InputValidationException, InputSchemaNotMatchingKeysException, exception_rules
import inspect

class Valipy:
    def __init__(self, input: Any = None) -> None:
        self.pipeline: list = []
        self.schemaModel: dict = {}


    def __pushToPipeline(self, rule: str, lamb:Callable = None) -> None:
        self.pipeline.append({"rule": rule, "lamb": lamb})
  
    # validation strategies
    def validate(self, input: Any = None) -> bool:
        for validator in self.pipeline:
            if validator['lamb'](input) == False:
                return False
        return True


    def tryValidate(self, input: Any = None) -> Union[Any, InputValidationException]:
        for validator in self.pipeline:
            if validator['lamb'](input) == False:
                if validator["rule"] in exception_rules:
                    raise exception_rules[validator["rule"]](input = input,message=f"{str(input)} failed at rule {validator['rule']}" ,rule=validator["rule"])
                else:
                    raise InputValidationException(input = input, rule=validator["rule"])
        return input
    
    # schema validation
    def __validateSchemaStrict(self, inputData: dict = None) -> Union[bool, InputSchemaNotMatchingKeysException]:
        # check that the input matches exactly the schema keys
        if len(inputData) != len(self.schemaModel):
            raise InputSchemaNotMatchingKeysException(input = inputData,message='Schema does not match' ,rule="schemaStrict")
        for key in inputData:
            if key not in self.schemaModel:
                raise InputSchemaNotMatchingKeysException(input = inputData,message='Schema does not match', rule="schemaStrict")
            



    def isSchema(self, schema: dict[Self]) -> Self:
        self.schemaModel = schema
        return self

    
    def validateSchema(self,data:dict = {}, strict: bool = False, useExcept: bool = False) -> bool:
        if strict == True:
            try:
                self.__validateSchemaStrict(inputData=data)
            except InputSchemaNotMatchingKeysException:
                return False
        # loop the schema
        for key, val in self.schemaModel.items():
            # if the key is not in the input data return false
            if key not in data:
                pass
            # if the key is in the input data but the value is not valid return false
            if useExcept == True:
                return val.tryValidate(data[key])
            else:

                return val.validate(data[key])
        
            
    def isValidPattern(self, pattern: Literal) -> Self:
        import re
        def lamb(x):
            if re.match(pattern, x):
                return True
            return False
        self.__pushToPipeline("isValidPattern", lamb)
        return self
    # validation rules
    def isEqual(self, val: Any) -> Self:
        def lamb(x):
            if x != val:
                return False
            return True

        self.__pushToPipeline("isEqual", lamb)
        return self

    def isNumber(self) -> Self:
        def lamb(x):
            if type(x) != int and type(x) != float:
                return False
            return True

        self.__pushToPipeline("isNumber", lamb)
        return self

    def isNumeric(self):
        # if input is a string try to convert to number
        def lamb(x):
            try:
                float(x)
                return True
            except Exception as e:
                try:
                    int(x)
                    return True
                except ValueError:
                    return False

        self.__pushToPipeline("isNumeric", lamb)
        return self
    
        

    def isNone(self) -> Self:
        def lamb(x):
            if x == None:
                return True
            return False
        self.__pushToPipeline("isNone", lamb)
        return self

    def isLowerCase(self) -> Self:
        def lamb(x):
                

            if type(x) != str:
                return False

            if x.islower()== True: 
                return True
            else:
                return False
        self.__pushToPipeline("isNone", lamb)
        return self

    def isUpperCase(self) -> Self:
        def lamb(x):
            if type(x) != str:
                return False
            if x.isupper() == True:
                return True
            else:
                return False
        self.__pushToPipeline("isUpperCase", lamb)
        return self
    def isCapitalized(self) -> Self:
        def lamb(x):
            if type(x) != str:
                return False
            if x[0].isupper() == True:
                return True
            else:
                return False
        self.__pushToPipeline("isCapitalized", lamb)
        return self
    def isEveryWordCapitalized(self) -> Self:

        def lamb(x):
            if type(x) != str:
                return False
            for word in x.split():
                if not word[0].isupper():
                    return False
            return True
        self.__pushToPipeline("isEveryWordCapitalized", lamb)
        return self
    def atStart(self, val: Any):
        def lamb(x):
            try:
                len(x)
            except TypeError:
                return False
            if x[0] != val:
                return False
            return True
        self.__pushToPipeline("atStart", lamb)
        return self

    def atEnd(self, val: Any):
        def lamb(x):
            try:
                len(x)
            except TypeError:
                return False
            if x[-1] != val:
                return False
            return True
        self.__pushToPipeline("atEnd", lamb)
        return self

    def isEmptyString(self) -> Self:
        def lamb(x):
            if type(x) != str:
                return False
            if len(x) > 0:
                return False
            return True
        self.__pushToPipeline("isEmptyString", lamb)    
        return self

   
    def isEmpty(self) -> Self:
        def lamb(x):
            try:
                len(x)
            except TypeError:
                return False
            if len(x) > 0:
                return False
            return True
        self.__pushToPipeline("isEmpty", lamb)
        return self
   

    def isMinLength(self, val: int) -> Self:
        def lamb(x):
            try:
                len(x)
            except TypeError:
                return False
            if len(x) < val:
                return False
            return True
        self.__pushToPipeline("isMinLength", lamb)
        return self


    def isMaxLength(self, val: int) -> Self:
        def lamb(x):
            try:
                len(x)
            except TypeError:
                return False
            if len(x) > val:
                return False
            return True
        self.__pushToPipeline("isMaxLength", lamb)
        return self
      
    
    def isInBetweenLength(self, min: int, max: int) -> Self:
            def lamb(x):
                try:
                    len(x)
                except TypeError:
                    return False
                if len(x) < min or len(x) > max:
                    return False
                return True
            self.__pushToPipeline("isInBetweenLength", lamb)
            return self
    
    

    def isInBetween(self, min: int, max: int) -> Self:
        def lamb(x):
            if type(x) != int and type(x) != float:
                return False
            if x <= min or x >= max:
                return False
            return True
        self.__pushToPipeline("isInBetween", lamb)
        return self
    

    def isLessThan(self, val: int) -> Self:
        def lamb(x):
            if type(x) != int and type(x) != float:
                return False
            if x >= val:
                return False
            return True
        self.__pushToPipeline("isLessThan", lamb)
        return self
    

   

    def isGreaterThan(self, val: int) -> Self:
        def lamb(x):
            if type(x) != int and type(x) != float:
                return False
            if x <= val:
                return False
            return True
        self.__pushToPipeline("isGreaterThan", lamb)
        return self
    

   

    def isNegative(self) -> Self:
        def lamb(x):
            if type(x) != int and type(x) != float:
                return False
            if x >= 0:
                return False
            return True
        self.__pushToPipeline("isNegative", lamb)
        return self

    def isPositive(self) -> Self:
        def lamb(x):
            if type(x) != int and type(x) != float:
                return False
            if x <= 0:
                return False
            return True
        self.__pushToPipeline("isPositive", lamb)
        return self


    def isEven(self) -> Self:
        def lamb(x):
            if type(x) != int:
                return False
            if x % 2 != 0:
                return False
            return True
        self.__pushToPipeline("isEven", lamb)
        return self
      

    def isOdd(self) -> Self:
        def lamb(x):
            if type(x) != int:
                return False
            if x % 2 == 0:
                return False
            return True
        self.__pushToPipeline("isOdd", lamb)
        return self
    

    def itIncludes(self, val: Any):
        def lamb(x):
            if type(x) != list or type(x) != str:
                return False
            if val not in x:
                return False
            return True
        self.__pushToPipeline("itIncludes", lamb)
        return self
    
    def itIncludesNumbers(self):
        def lamb(x):
            if type(x) == str:
                for char in x:
                    if char.isdigit():
                        return True
                return False
            if type(x) == list or type(x) == set:
                for item in x:
                    if type(item) == int or type(item) == float:
                        return True
                return False
        self.__pushToPipeline("itIncludesNumbers", lamb)
        return self
    
    def isInt(self):
        def lamb(x):
            if type(x) != int:
                return False
            return True
        self.__pushToPipeline("isInt", lamb)
        return self
    

    def isFloat(self):
        def lamb(x):
            if type(x) != float:
                return False
            return True
        self.__pushToPipeline("isFloat", lamb)
        return self
    

    def itHasSpaces(self):
        def lamb(x):
            if type(x) != str:
                return False
            if " " not in x:
                return False
            return True
        self.__pushToPipeline("itHasSpaces", lamb)
        return self
    

    def itPassedSome(self):
        def lamb(x):
            for validator in self.pipeline:
                if validator["passed"] == True:
                    return True
            return False
        self.__pushToPipeline("itPassedSome", lamb)
        return self

    def isInstance(self, val: Type) -> Self:
        def lamb(x):
            if not isinstance(x, val):
                return False
            return True
        self.__pushToPipeline("isInstance", lamb)
        return self
    
    def isType(self, val:Type) -> Self:
        def lamb(x):
            if type(x) != val:
                return False
            return True
        self.__pushToPipeline("isType", lamb)
        return self



from typing import Any


class InputValidationException(Exception):
    def __init__(
        self, input: Any, message: str = "Input is not valid", rule: str = "", natural_language_message: str = ""
    ) -> None:
        self.input = input
        self.message = message
        self.rule = rule
        self.natural_language_message = natural_language_message
        super().__init__(self.message)

    
    def to_dict(self) -> dict:
        return {
            "input": self.input,
            "message": self.message,
            "rule": self.rule,
            "natural_language_message":self.natural_language_message
        }
    def __str__(self) -> str:
        return self.message



class InputSchemaNotMatchingKeysException(InputValidationException):
    def __init__(
        self, input: Any, message: str = "Input is not valid", rule: str = "", natural_language_message: str = ""
    ) -> None:
        super().__init__(input, message, rule, natural_language_message)
    

# specific Exceptions


class InputIsEqualException(InputValidationException):
     def __init__(
        self, input: Any, message: str = "Input is not valid", rule: str = "", natural_language_message: str = "the input is not equal to the specified value"
    ) -> None:
        super().__init__(input, message, rule, natural_language_message)

class InputIsInstanceException(InputValidationException):
     def __init__(
        self, input: Any, message: str = "Input is not valid", rule: str = "", natural_language_message: str = "the input is not an instance of the specified class"
    ) -> None:
        super().__init__(input, message, rule, natural_language_message)

class InputIsTypeException(InputValidationException):
     def __init__(
        self, input: Any, message: str = "Input is not valid", rule: str = "", natural_language_message: str = "the input is not of the specified type"
    ) -> None:
        super().__init__(input, message, rule, natural_language_message)

class InputIsNumberException(InputValidationException):
     def __init__(
        self, input: Any, message: str = "Input is not valid", rule: str = "", natural_language_message: str = "the input is not a number"
    ) -> None:
        super().__init__(input, message, rule, natural_language_message)

class InputIsNumericException(InputValidationException):
     def __init__(
        self, input: Any, message: str = "Input is not valid", rule: str = "", natural_language_message: str = "the input is not numeric"
    ) -> None:
        super().__init__(input, message, rule, natural_language_message)

class InputIsNoneException(InputValidationException):
     def __init__(
        self, input: Any, message: str = "Input is not valid", rule: str = "", natural_language_message: str = "the input is None"
    ) -> None:
        super().__init__(input, message, rule, natural_language_message)

class InputIsLowerException(InputValidationException):
     def __init__(
        self, input: Any, message: str = "Input is not valid", rule: str = "", natural_language_message: str = "the input is not fully lower case"
    ) -> None:
        super().__init__(input, message, rule, natural_language_message)

class InputIsUpperException(InputValidationException):
     def __init__(
        self, input: Any, message: str = "Input is not valid", rule: str = "", natural_language_message: str = "the input is not fully upper case"
    ) -> None:
        super().__init__(input, message, rule, natural_language_message)
class InputIsCapitalizedException(InputValidationException):
    def __init__(
        self, input: Any, message: str = "Input is not valid", rule: str = "", natural_language_message: str = "the input is not capitalized"
    ) -> None:
        super().__init__(input, message, rule, natural_language_message)
class InputIsEveryWordCapitalizedException(InputValidationException):
    def __init__(
        self, input: Any, message: str = "Input is not valid", rule: str = "", natural_language_message: str = "the input is not capitalized at every word"
    ) -> None:
        super().__init__(input, message, rule, natural_language_message)
class InputAtStartException(InputValidationException):
     def __init__(
        self, input: Any, message: str = "Input is not valid", rule: str = "", natural_language_message: str = "the input does not start with the specified value"
    ) -> None:
        super().__init__(input, message, rule, natural_language_message)

class InputAtEndException(InputValidationException):
     def __init__(
        self, input: Any, message: str = "Input is not valid", rule: str = "", natural_language_message: str = "the input does not end with the specified value"
    ) -> None:
        super().__init__(input, message, rule, natural_language_message)

class InputIsEmptyStringException(InputValidationException):
     def __init__(
        self, input: Any, message: str = "Input is not valid", rule: str = "", natural_language_message: str = "the input is empty"
    ) -> None:
        super().__init__(input, message, rule, natural_language_message)

class InputIsEmptyException(InputValidationException):
     def __init__(
        self, input: Any, message: str = "Input is not valid", rule: str = "", natural_language_message: str = "the input is empty"
    ) -> None:
        super().__init__(input, message, rule, natural_language_message)


class InputIsMinLengthException(InputValidationException):
     def __init__(
        self, input: Any, message: str = "Input is not valid", rule: str = "", natural_language_message: str = "the length of the input is less than the specified value"
    ) -> None:
        super().__init__(input, message, rule, natural_language_message)

class InputIsMaxLengthException(InputValidationException):
     def __init__(
        self, input: Any, message: str = "Input is not valid", rule: str = "", natural_language_message: str = "the length of the input is greater than the specified value"
    ) -> None:
        super().__init__(input, message, rule, natural_language_message)


class InputIsInBetweenLengthException(InputValidationException):
     def __init__(
        self, input: Any, message: str = "Input is not valid", rule: str = "", natural_language_message: str = "the length of the input is not in between the specified values"
    ) -> None:
        super().__init__(input, message, rule, natural_language_message)


class InputIsLessThanException(InputValidationException):
     def __init__(
        self, input: Any, message: str = "Input is not valid", rule: str = "", natural_language_message: str = "the input is less than the specified value"
    ) -> None:
        super().__init__(input, message, rule, natural_language_message)


class InputIsGreaterThanException(InputValidationException):
     def __init__(
        self, input: Any, message: str = "Input is not valid", rule: str = "", natural_language_message: str = "the input is greater than the specified value"
    ) -> None:
        super().__init__(input, message, rule, natural_language_message)

class InputIsInBetweenException(InputValidationException):
     def __init__(
        self, input: Any, message: str = "Input is not valid", rule: str = "", natural_language_message: str = "the input is not in between the specified values"
    ) -> None:
        super().__init__(input, message, rule, natural_language_message)

class InputIsEvenException(InputValidationException):
     def __init__(
        self, input: Any, message: str = "Input is not valid", rule: str = "", natural_language_message: str = "the input is not even"
    ) -> None:
        super().__init__(input, message, rule, natural_language_message)

class InputIsNegativeException(InputValidationException):
     def __init__(
        self, input: Any, message: str = "Input is not valid", rule: str = "", natural_language_message: str = "the input is not negative"
    ) -> None:
        super().__init__(input, message, rule, natural_language_message)
class InputIsPositiveException(InputValidationException):
     def __init__(
        self, input: Any, message: str = "Input is not valid", rule: str = "", natural_language_message: str = "the input is not positive"
    ) -> None:
        super().__init__(input, message, rule, natural_language_message)

class InputIsOddException(InputValidationException):
     def __init__(
        self, input: Any, message: str = "Input is not valid", rule: str = "", natural_language_message: str = "the input is not odd"
    ) -> None:
        super().__init__(input, message, rule, natural_language_message)

class InputItIncludesException(InputValidationException):
     def __init__(
        self, input: Any, message: str = "Input is not valid", rule: str = "", natural_language_message: str = "the input does not include the specified value"
    ) -> None:
        super().__init__(input, message, rule, natural_language_message)

class InputItIncludeNumbersException(InputValidationException):
     def __init__(
        self, input: Any, message: str = "Input is not valid", rule: str = "", natural_language_message: str = "the input does not include numbers"
    ) -> None:
        super().__init__(input, message, rule, natural_language_message)

class InputIsIntException(InputValidationException):
     def __init__(
        self, input: Any, message: str = "Input is not valid", rule: str = "", natural_language_message: str = "the input is not a whole number"
    ) -> None:
        super().__init__(input, message, rule, natural_language_message)

class InputIsFloatException(InputValidationException):
     def __init__(
        self, input: Any, message: str = "Input is not valid", rule: str = "", natural_language_message: str = "the input is not a decimal number"
    ) -> None:
        super().__init__(input, message, rule, natural_language_message)

class InputItHasSpacesException(InputValidationException):
     def __init__(
        self, input: Any, message: str = "Input is not valid", rule: str = "", natural_language_message: str = "the input does not have spaces"
    ) -> None:
        super().__init__(input, message, rule, natural_language_message)

class InputItPassedSomeException(InputValidationException):
     def __init__(
        self, input: Any, message: str = "Input is not valid", rule: str = "", natural_language_message: str = "all the validation rules for the input did not pass"
    ) -> None:
        super().__init__(input, message, rule, natural_language_message)



# exception rules
exception_rules: dict = {
    "isEqual": InputIsEqualException,
    "isInstance": InputIsInstanceException,
    "isType": InputIsTypeException,
    "isNumber": InputIsNumberException,
    "isNumeric": InputIsNumericException,
    "isNone": InputIsNoneException,
    "isLower": InputIsLowerException,
    "isCapitalized": InputIsCapitalizedException,
    "isEveryWordCapitalized": InputIsEveryWordCapitalizedException,
    "isUpper": InputIsUpperException,
    "atStart": InputAtStartException,
    "atEnd": InputAtEndException,
    "isEmptyString": InputIsEmptyStringException,
    "isEmpty": InputIsEmptyException,
  
    "isMaxLength": InputIsMaxLengthException,
    "isInBetweenLength": InputIsInBetweenLengthException,
    
    "isLessThan": InputIsLessThanException,
    "isGreaterThan": InputIsGreaterThanException,
    "isNegative": InputIsNegativeException,
    "isPositive": InputIsPositiveException,
    "isInBetween": InputIsInBetweenException,

    "isEven": InputIsEvenException,
    "isOdd": InputIsOddException,
    "itIncludes": InputItIncludesException,
    "itIncludeNumbers": InputItIncludeNumbersException,
    "isInt": InputIsIntException,
    "isFloat": InputIsFloatException,
    "itHasSpaces": InputItHasSpacesException,
    "itPassedSome": InputItPassedSomeException,
}



from valipy import Valipy, InputValidationException, InputIsNumericException, InputIsEqualException, InputIsInBetweenLengthException
personModel = {
    "name": Valipy().isType(str).isInBetweenLength(3, 20),
    "age": Valipy().isType(int).isInBetween(18, 100),
    

}

testData = {
    "name": "t",
	"age": 20,
}
# validate
try:
	x = Valipy().isSchema(personModel).validateSchema(testData, strict=True, useExcept=True)

except InputIsNumericException as e:
	print(e)

except InputIsInBetweenLengthException as e:
	print(e)
	print(e.to_dict())


from setuptools import find_packages, setup
import os

description = "A chainable, fluent Python library for validating data"
def read_file(filename):
    try:
        with open(os.path.join(os.path.dirname(__file__), filename)) as file:
            return file.read()
    except Exception as e:
        print(f"Error reading {filename}: {e}")
        return description
try:
    long_description = read_file('readme.md') 
except Exception as e:
    long_description = description
setup(
    name='valipy',
    packages=find_packages(),
    version='0.2.5',
    description='A chainable, fluent Python library for validating data',
    long_description_content_type='text/markdown',
    long_description=long_description,

    author='Joaquin Jose Von Chong',
    license='MIT',
    author_email='jjvonchong@outlook.com',
    url='https://github.com/pimepan/valipy',
    keywords=['validation', 'data', 'schema', 'schema validation', 'regex', 'pattern matching'],
    install_requires=["twine", "wheel", "setuptools"],
    classifiers=[
        'Development Status :: 4 - Beta',
        'Intended Audience :: Developers',
        'Topic :: Software Development',
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python :: 3.11',
        'Programming Language :: Python :: 3.12',
    ],
)
